      program dmain

      character*10 fread, fwrite
      parameter (fread='vmlm.in',fwrite='vmlm.out')
      integer npmax, nmax, mmax, lwamax
      parameter (nmax=160000,mmax=100)
      parameter (npmax=25,lwamax=2*nmax*mmax+2*nmax+mmax)
      integer nread, nwrite
      parameter (nread=1,nwrite=2)
c     **********
c
c     This is a sample driver for the minimization test problems
c     included in MINPACK-2.
c
c     Problem parameters are read from the data file fread named
c     in the parameter statement above.
c     Information on the performance of the algorithm is printed
c     to the data file fwrite named in the parameter statement above.
c
c     This is only a sample driver, many other drivers are possible.
c
c     Subprograms called
c
c       USER ........... dminfg
c
c       MINPACK-2 ...... dvmlm, dtimer
c
c       Level 1 BLAS ... dnrm2
c
c     MINPACK-2 Project. June 1996.
c     Argonne National Laboratory.
c     Brett M. Averick and Jorge J. More'.
c
c     **********
      double precision ten
      parameter (ten=10.0d0)

      character*6 prob
      character*6 probs(npmax)
      character*60 tasks(npmax)
      integer i, ic, iter, k, nx, ny, m, nfev, ngev, ntries
      integer ns(npmax), nfevs(npmax), ngevs(npmax), iters(npmax)
      double precision gnorm1, gnorm2, f1, f2, par, stime, ftime, xsf
      double precision pars(npmax), time(npmax), fs(npmax), gs(npmax)
      double precision wa1(2*nmax)

      character*60 task
      integer n, maxfev
      integer isave(5)
      double precision f, frtol, fatol, fmin
      double precision x(nmax), g(nmax), wa(lwamax)
      double precision dsave(24)

      double precision dnrm2
      external dminfg, dvmlm, dtimer, dnrm2

      open (nread,file=fread,status='old')
      open (nwrite,file=fwrite)

c     Read frtol,fatol,fmin, m.

      read (nread,*) frtol, fatol, fmin, m
      write (*,*) frtol, fatol, fmin, m

      ic = 0
   10 continue

c     Read in problem parameters.

      read (nread,*) prob, n, nx, ny, ntries, par
      write (*,*) prob, n, par

      if (prob .eq. 'STOP') go to 50

      do 40 k = 1, ntries

c        Get the initial approximation xs.

         task = 'XS'
         call dminfg(n,nx,ny,x,f,g,task,prob,par,wa1)
         if (prob .eq. 'ERROR') then
            go to 50
         else
            ic = ic + 1
         end if

c        If desired scale xs.

         if (k .gt. 1) then
            xsf = ten**(k-1)
            do 20 i = 1, n
               x(i) = xsf*x(i)
   20       continue
         end if

c        Evaluate the function and the gradient at xs.

         task = 'FG'
         call dminfg(n,nx,ny,x,f,g,task,prob,par,wa1)

c        Store the function value and the gradient norm at xs.

         f1 = f
         gnorm1 = dnrm2(n,g,1)

c        Initialize.

         nfev = 0
         ngev = 0
         iter = 0
         maxfev = 10000

c        Call and time the algorithm.

         call dtimer(stime)

         task = 'START'
   30    continue

         call dvmlm(n,x,f,g,frtol,fatol,fmin,task,m,wa(1),wa(n*m+1),
     +              wa(2*n*m+1),isave,dsave,wa(2*n*m+m+1),
     +              wa(2*n*m+m+n+1))

         if (task .eq. 'FG') then
            nfev = nfev + 1
            ngev = ngev + 1
            call dminfg(n,nx,ny,x,f,g,task,prob,par,wa1)
            if (nfev .lt. maxfev) go to 30
            task = 'STOP: MAXFEV LIMIT REACHED'
         end if
         if (task .eq. 'NEWX') then
            iter = iter + 1
            go to 30
         end if

         call dtimer(ftime)

c        Store the final function value and gradient norm.

         f2 = f
         gnorm2 = dnrm2(n,g,1)

c        Record information on the algorithm.

         probs(ic) = prob
         ns(ic) = n
         pars(ic) = par
         iters(ic) = iter
         nfevs(ic) = nfev
         ngevs(ic) = ngev
         tasks(ic) = task
         fs(ic) = f2
         gs(ic) = gnorm2
         time(ic) = ftime - stime
         write (nwrite,1000) prob, n
         write (nwrite,2000) f1, f2, gnorm1, gnorm2, iter, nfev, nfev,
     +      time(ic), task
   40 continue

      go to 10

   50 continue

c     Write a summary of the information.

      write (nwrite,3000) ic
      do 60 i = 1, ic
         write (nwrite,4000) probs(i), ns(i), pars(i), iters(i),
     +      nfevs(i), ngevs(i), fs(i), gs(i), time(i), tasks(i) (1:4)
   60 continue

      close (nread)
      close (nwrite)

      stop

 1000 format (1x,'Problem ',a6,5x,'Dimension',i8//)
 2000 format (1x,'Initial function value     ',10x,d17.10//1x,
     +       'Final function value       ',10x,d17.10//1x,
     +       'Initial gradient 2 norm    ',10x,d17.10//1x,
     +       'Final gradient 2 norm      ',10x,d17.10//1x,
     +       'Number of iterations            ',10x,i10//1x,
     +       'Number of function evaluations  ',10x,i10//1x,
     +       'Number of gradient evaluations  ',10x,i10//1x,
     +       'Total Time                      ',9x,d11.3//1x,
     +       'Exit task ',a60)
 3000 format (////'Summary of ',i3,' calls to dvmlm: '//
     +       'prob      n     par   iters  nfev  ngev',6x,'f',9x,
     +       ' ||g||',4x,'time',6x,'task'/80('-'))
 4000 format (a6,i6,d9.1,3i6,1x,d12.4,d9.1,d12.4,a6)
      end

